// src/HighlightPopover.tsx
import {
  useRef,
  useMemo,
  useState,
  useEffect,
  useCallback,
  createContext,
  useContext
} from "react";
import { jsx, jsxs } from "react/jsx-runtime";
var HighlightPopoverContext = createContext(null);
function useHighlightPopover() {
  const context = useContext(HighlightPopoverContext);
  if (!context) {
    throw new Error(
      "useHighlightPopover must be used within a HighlightPopover"
    );
  }
  return context;
}
function HighlightPopover({
  children,
  renderPopover,
  className = "",
  offset = { x: 0, y: 0 },
  zIndex = 40,
  minSelectionLength = 1,
  onSelectionStart,
  onSelectionEnd,
  onPopoverShow,
  onPopoverHide
}) {
  const [showPopover, setShowPopover] = useState(false);
  const [popoverPosition, setPopoverPosition] = useState({
    top: 0,
    left: 0
  });
  const [currentSelection, setCurrentSelection] = useState("");
  const containerRef = useRef(null);
  const isSelectionWithinContainer = useCallback((selection) => {
    if (!containerRef.current) return false;
    for (let i = 0; i < selection.rangeCount; i++) {
      const range = selection.getRangeAt(i);
      if (!containerRef.current.contains(range.startContainer) || !containerRef.current.contains(range.endContainer)) {
        return false;
      }
    }
    return true;
  }, []);
  const handleSelection = useCallback(() => {
    requestAnimationFrame(() => {
      if (!containerRef.current) return;
      const selection = window.getSelection();
      if (selection && selection.toString().trim().length >= minSelectionLength && isSelectionWithinContainer(selection)) {
        onSelectionStart?.();
        const range = selection.getRangeAt(0);
        const rect = range.getBoundingClientRect();
        const containerRect = containerRef.current.getBoundingClientRect();
        const top = rect.bottom - containerRect.top - (offset && (offset.y ?? 0));
        const left = rect.left - containerRect.left + rect.width / 2 + (offset && (offset.x ?? 0));
        setPopoverPosition({ top, left });
        setCurrentSelection(selection.toString());
        setShowPopover(true);
        onPopoverShow?.();
        onSelectionEnd?.(selection.toString());
      } else {
        setShowPopover(false);
        onPopoverHide?.();
      }
    });
  }, [
    isSelectionWithinContainer,
    minSelectionLength,
    offset,
    onSelectionStart,
    onSelectionEnd,
    onPopoverShow,
    onPopoverHide
  ]);
  useEffect(() => {
    document.addEventListener("selectionchange", handleSelection);
    return () => {
      document.removeEventListener("selectionchange", handleSelection);
    };
  }, [handleSelection]);
  useEffect(() => {
    const handleClickOutside = (event) => {
      if (containerRef.current && !containerRef.current.contains(event.target)) {
        setShowPopover(false);
        onPopoverHide?.();
      }
    };
    document.addEventListener("mousedown", handleClickOutside);
    return () => {
      document.removeEventListener("mousedown", handleClickOutside);
    };
  }, [onPopoverHide]);
  const contextValue = {
    showPopover,
    setShowPopover,
    popoverPosition,
    currentSelection,
    setCurrentSelection
  };
  const popoverStyle = useMemo(
    () => ({
      zIndex,
      width: "max-content",
      position: "absolute",
      transform: "translateX(-50%)",
      top: `${popoverPosition.top}px`,
      left: `${popoverPosition.left}px`
    }),
    [zIndex, popoverPosition.top, popoverPosition.left]
  );
  return /* @__PURE__ */ jsx(HighlightPopoverContext.Provider, { value: contextValue, children: /* @__PURE__ */ jsxs(
    "div",
    {
      ref: containerRef,
      style: { position: "relative" },
      className,
      children: [
        children,
        showPopover && /* @__PURE__ */ jsx("div", { style: popoverStyle, role: "tooltip", "aira-live": "polite", children: renderPopover({
          position: popoverPosition,
          selection: currentSelection
        }) })
      ]
    }
  ) });
}
export {
  HighlightPopover,
  useHighlightPopover
};
